/*
 * Project name:
     mesure courant RMS par capteur  effet hall
 * Copyright:
     (c) FOURNET Olivier 20/04/2018
 * Revision History:
     2018-04-20 : cration projet exemple
         
 * Description:
     mesure courant RMS par capteur  effet hall avec envoi des valeurs par port srie chaques secondes
     
 * Test configuration:
     MCU:             PIC18F4620
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39761c.pdf
     Oscillator:      HS, 8.0000 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:

 */

// capteur de courant
// Allegro - ACS750LCA-050  - https://www.allegromicro.com/~/media/Files/Datasheets/ACS750-050-Datasheet.ashx
// New Product : https://www.allegromicro.com/en/Products/Current-Sensor-ICs/Fifty-To-Two-Hundred-Amp-Integrated-Conductor-Sensor-ICs/ACS770.aspx
// Sensitivity Sens (Typ.) 40mV/A
// 0A = VCC/2 --> pour 5V --> 2.5V --> 1023/2 = 511.5
static float kA = 25.0; // kA = 1A / 0.04V = 25
static float kVref = 4.8828125e-3; // kVref = Vref(5V) / 1024
float an0, val;

char str[24];

void UART1_jump_a_line()
{
 UART1_Write(10);
 UART1_Write(13);
}

void InitTimer0()
{
 // See http://www.enmcu.com/software/timer0calculatorandcodegeneration
 // Timer0
 // Prescaler 1:2; TMR0 Preload = 100; Actual Interrupt Time : 156 us
 // 156 us * 128 = 19,968ms soit 128 chantillons par priodes de 50,08 Hz
 // for values.

 T0CON         = 0xC0;
 TMR0L         = 0x64;

  INTCON.TMR0IE = 1;    //enable the timer0 by setting TRM0IE flag
  INTCON.GIE=1;         //globle interrupt enable
}

unsigned long int t_;
//Timer1
//Prescaler 1:1; TMR1 Preload = 57536; Actual Interrupt Time : 1 ms
void InitTimer1()
{
  T1CON         = 0x01;
  TMR1IF_bit         = 0;

  TMR1H         = 0xF8;
  TMR1L         = 0x30;
  TMR1IE_bit         = 1;
  //INTCON         = 0xC0;
}

unsigned long int t_x;
long int adc0_moy_x;
long int ac2_moy_x;
long int ac2_sum_x;

int j, aff;
unsigned int k, k_;
int adc0, ac, adc0_moy;
long int adc0_sum, adc0_moy_50;
long int ac2, ac2_sum, ac2_sum_t, ac2_moy, ac2_moy_50;

void interrupt(void)
{
 if(TMR0IF_bit)  // interrupt handler for the timer0 overflow
 {
  PORTB = 255 ;
  // Reset interrupt flag
  INTCON.TMR0IF = 0;
  // Reset the timer count
  TMR0L         = 0x64;   // 156s

    adc0 = ADC_Read(0);   // Get 10-bit results of AD conversion
    ac = adc0 - adc0_moy;  // retrais de la valeur moyenne pour avoir les valeurs alternatives centr sur le 0
    ac2 = ac * ac;  // calcule de la valeur au carr

    // Moyenne sur 128 mesures (pour 1 priode de 50Hz)
    if(j == 127)
    {
     adc0_sum += adc0;
     adc0_moy = adc0_sum >> 7;
     adc0_sum = 0;
     
     ac2_sum += ac2;

     ac2_sum_t = ac2_sum;
     ac2_moy  = ac2_sum >> 7;
     ac2_sum = 0;

     j = 0;
     k++;
     adc0_moy_50 += adc0_moy;
     ac2_moy_50 += ac2_moy;
     if(k == 50)
     {
      // rcupration des valeurs  un instant tx avant qu'elle ne change
      t_x = t_;
      adc0_moy_x = adc0_moy_50;
      ac2_moy_x = ac2_moy_50;
      adc0_moy_50 = 0;
      ac2_moy_50 = 0;
      aff = 1;
      k_ = k;
      k = 0;
     }
     else aff = 0;
    }
    else
    {
     adc0_sum += adc0;
     ac2_sum += ac2;

     j++;
    }

    PORTB = 0 ;
 }
 
 if(TMR1IF_bit) // interrupt handler for the timer1 overflow
 {
    // Reset interrupt flag
    TMR1IF_bit = 0;
    
    PORTD = 1 ;

    TMR1H         = 0xF8;
    TMR1L         = 0x30;

    //Enter your code here
    t_++;
    Delay_us(100);
    PORTD = 0 ;
 }
}
//

//-------------- main

void main() 
{
  TRISA  = 0xFF;              // PORTA is input
  TRISB = 0;                  // PORTB is output
  TRISD = 0;                  // PORTD is output
  PORTA = 0 ;
  PORTB = 0 ;
  PORTD = 0 ;
  
  adc0_moy = 0;
  adc0_sum = 0;
  
  ac2_moy = 0;
  ac2_sum = 0;
  
  j = 0;
  k = 0;
  aff = 0;

  InitTimer0();
  t_ = 0;
  InitTimer1();
  
  ADCON1 = 0x82;  // Vref+ = VDD, Vref- = VSS, RA0  RA5 = ADCin
  //
  UART1_Init(9600);               // Initialize UART module at 9600 bps

  Delay_ms(1000);                  // Wait for to stabilize
  PORTB = 255 ;

  UART1_jump_a_line();

  UART1_Write_Text("Start, using mikroC Firmware on PIC18 (18F4620-8MHz-64K ROM) v1.0.0.0");
  UART1_jump_a_line();
  UART1_Write_Text("Build : " __TIME__);
  UART1_Write_Text(", " __DATE__);
  UART1_jump_a_line();
  //
  Delay_ms(5000);                  // Wait for read version
  PORTB = 0 ;
  
  while(1) 
  {                     // Endless loop
   if(aff == 1)  // affichage tous les 50 cycles
   {
    val = (float) k_;

    aff = 0;
    val = 1.0 / val;

    UART1_Write_Text("Time: ");
    FloatToStr(t_x * 0.001, str);
    UART1_Write_Text(str);
    UART1_Write_Text("s , ");
    UART1_Write_Text("ADC0 Moy(");
    IntToStr(adc0_moy, str);
    UART1_Write_Text(str);
    UART1_Write_Text(" ) ");
    // moyenne des K valeurs
    an0  = (float)adc0_moy_x * val;
    an0  *= kVref; // an0(V) = ADC*(Vref/1024)
    FloatToStr(an0, str);
    UART1_Write_Text(str);
    UART1_Write_Text("V, K:");
    LongToStr(k_, str);
    UART1_Write_Text(str);
    UART1_Write_Text(" , ");
    // moyenne des K valeurs
    val  = (float)ac2_moy_x * val;
    if(val < 3) val = 0; // annulation des valeurs faible parasite
    val = sqrt(val) * kVref * kA; // courant efficace dans le capteur en A
    FloatToStr(val, str);
    UART1_Write_Text(str);
    UART1_Write_Text("A eff, ");
    val = val * 226.0;
    FloatToStr(val, str);
    UART1_Write_Text(str);
    UART1_Write_Text("VA");
    UART1_jump_a_line();
   }
  }
}